#pragma once

#include <math.h>
#define  pi 3.1415926535897932384626433832795
const float B = 4/pi;
const float C = -4/(pi*pi);
const float P = 0.225;

// approximates a sinewave with 
class CSine
{
public:
	CSine(void);
	~CSine(void);

	float m_fSampleRate;	// fs
	float m_fModulo;		// modulo counter 0->1
	float m_fInc;			// phase inc = fo/fs
	float m_fFo;			// osc frequency

	inline void reset()
	{
		m_fModulo = 0.0;
		m_fInc = 0.0;
	}

	inline void setFo(float fo)
	{
		m_fFo = fo;

		// calculate increment
		m_fInc = m_fFo/m_fSampleRate;
	}

	// http://devmaster.net/posts/9648/fast-and-accurate-sine-cosine
	// input is -pi to +pi
	inline float parabolicSine(float x, bool bHighPrecision = true)
	{
		float y = B * x + C * x * abs(x);

		if(bHighPrecision)
			y = P * (y * abs(y) - y) + y;   

		return y;
	}

	inline float doOscillate()
	{
		if(m_fModulo >= 1.0) 
			m_fModulo -= 1.0; 

		// calculate angle
		float fAngle = m_fModulo*2.0*pi - pi;
		
		m_fModulo += m_fInc;

		// call the parabolicSine approximator
		return parabolicSine(-fAngle);	
	}

};
